/*----------------------------------------------------------------------------
 * Name:    ROSA_function.c
 * Purpose:    (-) 
 *          Russian Event OS      (ROSA-e)
 * Note(s):       AVR
 *----------------------------------------------------------------------------
 *    .   
 *        .    
 *    )))   " ".
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2016 RUAL
 *----------------------------------------------------------------------------*/
#include "ROSA.h"

const char ROSA_Port_R[] = "AVR-ATMEGA";

/*   - */
unsigned char ROSA_AtomReadByte(volatile unsigned char* s)
{
	ROSA_InterruptDisable(); /*    */
	return *s;
}

ROSA_BASE_TypeDef ROSA_AtomWriteByte(unsigned char s, volatile unsigned char* d)
{
	*d = s;
	ROSA_InterruptEnable();	/*   */ 
	return 0;
}

/*  AVR   ,
					   */
void ROSA_InitInt(IRQ_Priority_TypeDef p) 
{
	ROSA_AVR_SW_PENDDDR |= (1<<ROSA_AVR_SW_PENDBIT); //   

#if defined(ROSA_AVR_SW_PENDBIT_PORTD2) // INT0
	ROSA_AVR_SW_ENREG |= (1<<ROSA_AVR_SW_ENBIT);
	EICRA |= (1<<ISC01)|(0<<ISC00);

#elif defined(ROSA_AVR_SW_PENDBIT_PORTD3) // INT1
	ROSA_AVR_SW_ENREG |= (1<<ROSA_AVR_SW_ENBIT);
	EICRA |= (1<<ISC11)|(0<<ISC10);

#elif  (defined(ROSA_AVR_SW_PENDBIT_PORTC0)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC1)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC2)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC3)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC4)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC5)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC6)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC7)) // PORTCx
	ACSR = (1<<ACIE)|(1<<ACIS1)|(0<<ACIS0);

	ADCSRB	|= (1<<ACME);
	ADMUX = ROSA_AVR_SW_PENDBIT;

#elif defined(ROSA_AVR_SW_PENDBIT_PORTD7) // PORTD7 (AIN1)
	ACSR = (1<<ACIE)|(1<<ACIS1)|(0<<ACIS0);

	ADCSRB	&= ~(1<<ACME);
#else 
	#error "Bed ROSA_AVR_SW_PENDBIT in <ROSA_config.h>"
#endif	 
}

 
/*      _inline*/
#define pendSWI() {ROSA_AVR_SW_PENDREG |= (1<<ROSA_AVR_SW_PENDBIT);\
				  ROSA_AVR_SW_PENDREG &= ~(1<<ROSA_AVR_SW_PENDBIT);}

/*     ROSA_function.c */
extern Event_Number_TypeDef volatile	_LastRun_Event;
/*  ROSA_function.c */
extern ROSA_BASE_TypeDef Event_Planner(Event_Number_TypeDef n);
/*    ROSA_function.c */
extern ROSA_BASE_TypeDef Event_Beginner(void);	 
/*   ROSA_function.c */
extern Event_Number_TypeDef Event_Closer(Event_Number_TypeDef n, Event_Number_TypeDef l); 
/*       */
extern const Event_TypeDef Event_List [];


/*   */
void ROSA_CallEvent(Event_Number_TypeDef n)
{
	if (Event_Planner(n) != 255) 
					pendSWI();	
}


/*     */
ISR (ROSA_AVR_SW_VECT)
{
	Event_Number_TypeDef new_ev; 
	Event_Number_TypeDef old_ev; 

	do { /*    */
		new_ev = Event_Beginner();
		old_ev = _LastRun_Event;

		if (new_ev != 255) {
			ROSA_InterruptEnable();		/*   */
			Event_List[new_ev](); 	/*   
										   */
		}
	
		ROSA_InterruptDisable(); 		/*    */

		/*    -  */
		if (!(ROSA_AVR_SW_ENREG & (1<<ROSA_AVR_SW_ENBIT))) break;
		
	} /*  ,	 
	    -    */
	while (Event_Closer(new_ev, old_ev) != 255); 
}
