/*----------------------------------------------------------------------------
 * Name:    ROSA_port.h
 * Purpose:    (-) 
 *					Russian Event OS			(ROSA-e)
 * Note(s):        AVR
 *----------------------------------------------------------------------------
 *    .   
 *        .    
 *    )))   " ".
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2016 RUAL
 *----------------------------------------------------------------------------*/
 
#include "ROSA_config.h"

#ifndef __ROSA_PORT_H
#define __ROSA_PORT_H

#ifdef __cplusplus
 extern "C" {
#endif 
	 
typedef unsigned char ROSA_BASE_TypeDef;
typedef unsigned char IRQ_Priority_TypeDef;	 

#define ROSA_InterruptDisable() cli()
#define ROSA_InterruptEnable()	sei()


/*      */
#if defined(ROSA_AVR_SW_PENDBIT_PORTD2) // INT0
	#define ROSA_AVR_SW_VECT		INT0_vect 	/*      */
	#define ROSA_AVR_SW_ENREG		EIMSK  		/*       */
	#define ROSA_AVR_SW_ENBIT		INT0	 	/*       */
	#define ROSA_AVR_SW_PENDREG		PORTD 		/*       */
	#define ROSA_AVR_SW_PENDDDR		DDRD 		/*       */
	#define ROSA_AVR_SW_PENDBIT		PORTD2 		/*       */

#elif defined(ROSA_AVR_SW_PENDBIT_PORTD3) // INT1
	#define ROSA_AVR_SW_VECT		INT1_vect 	/*      */
	#define ROSA_AVR_SW_ENREG		EIMSK  		/*       */
	#define ROSA_AVR_SW_ENBIT		INT1	 	/*       */
	#define ROSA_AVR_SW_PENDREG		PORTD 		/*       */
	#define ROSA_AVR_SW_PENDDDR		DDRD 		/*       */
	#define ROSA_AVR_SW_PENDBIT		PORTD3 		/*       */

#elif ( defined(ROSA_AVR_SW_PENDBIT_PORTC0)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC1)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC2)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC3)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC4)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC5)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC6)||\
		defined(ROSA_AVR_SW_PENDBIT_PORTC7)) // PORTCx
	#define ROSA_AVR_SW_VECT		ANALOG_COMP_vect /*      */
	#define ROSA_AVR_SW_ENREG		ACSR  		/*       */
	#define ROSA_AVR_SW_ENBIT		ACIE	 	/*       */
	#define ROSA_AVR_SW_PENDREG		PORTC 		/*       */
	#define ROSA_AVR_SW_PENDDDR		DDRC 		/*       */
	
	#ifdef ROSA_AVR_SW_PENDBIT_PORTC0 /*       */
		#define ROSA_AVR_SW_PENDBIT		PORTC0 			 
	#elif defined(ROSA_AVR_SW_PENDBIT_PORTC1) 
		#define ROSA_AVR_SW_PENDBIT		PORTC1
	#elif defined(ROSA_AVR_SW_PENDBIT_PORTC2) 
		#define ROSA_AVR_SW_PENDBIT		PORTC2
	#elif defined(ROSA_AVR_SW_PENDBIT_PORTC3) 
		#define ROSA_AVR_SW_PENDBIT		PORTC3
	#elif defined(ROSA_AVR_SW_PENDBIT_PORTC4) 
		#define ROSA_AVR_SW_PENDBIT		PORTC4
	#elif defined(ROSA_AVR_SW_PENDBIT_PORTC5) 
		#define ROSA_AVR_SW_PENDBIT		PORTC5
	#elif defined(ROSA_AVR_SW_PENDBIT_PORTC6) 
		#define ROSA_AVR_SW_PENDBIT		PORTC6
	#elif defined(ROSA_AVR_SW_PENDBIT_PORTC7) 
		#define ROSA_AVR_SW_PENDBIT		PORTC7
	#endif

#elif defined(ROSA_AVR_SW_PENDBIT_PORTD7) // PORTD7 (AIN1)
	#define ROSA_AVR_SW_VECT		ANALOG_COMP_vect /*      */
	#define ROSA_AVR_SW_ENREG		ACSR  		/*       */
	#define ROSA_AVR_SW_ENBIT		ACIE	 	/*       */
	#define ROSA_AVR_SW_PENDREG		PORTD 		/*       */
	#define ROSA_AVR_SW_PENDDDR		DDRD 		/*       */
	#define ROSA_AVR_SW_PENDBIT		PORTD7 		/*       */
#else
	#error "Bed ROSA_AVR_SW_PENDBIT in <ROSA_config.h>"

#endif


void ROSA_InitInt(IRQ_Priority_TypeDef p);	 
	 
extern unsigned char ROSA_AtomReadByte(volatile unsigned char* s);
extern ROSA_BASE_TypeDef ROSA_AtomWriteByte(unsigned char s,volatile unsigned char* d);

/*    - 
	     */
#define ROSA_Critical_Section_Begin()	ROSA_AtomWriteByte(ROSA_AtomReadByte(&ROSA_AVR_SW_ENREG)\
													&~(1<<ROSA_AVR_SW_ENBIT),&ROSA_AVR_SW_ENREG)
#define ROSA_Critical_Section_End()   	ROSA_AtomWriteByte(ROSA_AtomReadByte(&ROSA_AVR_SW_ENREG)\
													|(1<<ROSA_AVR_SW_ENBIT),&ROSA_AVR_SW_ENREG)
	 
#ifdef __cplusplus
}
#endif

#endif /* __ROSA_PORT_H */
