/*----------------------------------------------------------------------------
 * Name:    ROSA_function.c
 * Purpose:    (-) 
 *					Russian Event OS			(ROSA-e)
 * Note(s):     
 *----------------------------------------------------------------------------
 *    .   
 *        .    
 *    )))   " ".
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2016 RUAL
 *----------------------------------------------------------------------------*/
#include "ROSA.h"

/*  */
const char ROSA_Name_R[] = "   (-)";
const char ROSA_Name_E[] = "Russian Event OS (ROSA-e)";
const char ROSA_Realise_R[] = ". 1.0";
const char ROSA_Realise_E[] = "ver  1.0";
#define    ROSA_Port_E   ROSA_Port_R


Event_Stat_TypeDef 			_Ev_Stat[Event_Count];  		/*    */
#ifndef ROSA_ConstPriority
Event_Priorety_TypeDef 	_Ev_Prior[Event_Count]; 			/*    */
#else 
extern const Event_Priorety_TypeDef _Ev_Prior[]; 			/*       */
#endif

Event_Number_TypeDef	 	_Ev_FailCount[Event_Count]; 	/*    */

Event_Number_TypeDef volatile	_PreRun_Event;						/*  -   */
Event_Number_TypeDef volatile	_CurRun_Event;						/*    */
Event_Number_TypeDef volatile	_LastRun_Event;						/*    */

Event_Number_TypeDef 		_CountWait_Event; 							/*     */

/*  */
#ifndef ROSA_ConstPriority
void SetEventPriority(Event_Number_TypeDef n, Event_Priorety_TypeDef p)
{
	_Ev_Prior[n] = p;
}
#endif

Event_Priorety_TypeDef GetEventPriority(Event_Number_TypeDef n)
{
	return  _Ev_Prior[n];
}
/*   */
Event_Stat_TypeDef ROSA_GetStatEvent(Event_Number_TypeDef n)
{
	return _Ev_Stat[n];
}

/*      */
Event_Number_TypeDef ROSA_GetFailCount(Event_Number_TypeDef n)
{
	return _Ev_FailCount[n];
}

void ROSA_ResetFailCount(Event_Number_TypeDef n)
{
	_Ev_FailCount[n] = 0;
}

/*    */
void ROSA_BlockEvent(Event_Number_TypeDef n)
{
	Event_Stat_TypeDef state;
		
	do {
		state = (Event_Stat_TypeDef)ROSA_AtomReadByte(&_Ev_Stat[n]);
		
		switch(state){			
			case	EVENT_READY:			/*   ?! */
				ROSA_InterruptDisable();
				_PreRun_Event = 255;	/*   ! */							
			
			default:
				state = EVENT_BLOCK;			
				break;
			
			case	EVENT_BLOCK: 			/*   */		
			case	EVENT_BLOCK_WAIT: /*     */
				break;								/*    */
		}
		
	} while (ROSA_AtomWriteByte(state, &_Ev_Stat[n]));
	
	ROSA_InterruptEnable();
}


void ROSA_UnBlockEvent(Event_Number_TypeDef n)
{
	ROSA_BASE_TypeDef  ltch = 0;
	Event_Stat_TypeDef state;
		
	do {
		state = (Event_Stat_TypeDef)ROSA_AtomReadByte(&_Ev_Stat[n]);
		
		switch(state){		
			
			case	EVENT_BLOCK_WAIT: 	/*     */
				ltch = 1;								/*  ! */		
			
			case	EVENT_BLOCK:
				state = EVENT_STOP;		
			
			default:					
				break;			
		}
		
	} while (ROSA_AtomWriteByte(state, &_Ev_Stat[n]));
	
	if (ltch) ROSA_CallEvent(n);
}
	


/************************   ********************************/

/*   */
void __inline _FailCount(Event_Number_TypeDef n)
{
	if (_Ev_FailCount[n] != 255) 
		_Ev_FailCount[n] += 1;
}

/*      */
#ifndef ROSA_WaitEvent_QUEUE /*   */
Event_Number_TypeDef volatile	_Scan_Wait_Event; 				/*    */
Event_Priorety_TypeDef	_MaxWait_Event_Prior; 					/*     */

void _SetWaitEvent(Event_Number_TypeDef n)
{
	_Ev_Stat[n] = EVENT_WAIT;
	
	if (_Scan_Wait_Event > n ) _Scan_Wait_Event = n; //    
	
	if (_MaxWait_Event_Prior > _Ev_Prior[n])	{		 
		 _MaxWait_Event_Prior = _Ev_Prior[n];
	}
	_CountWait_Event += 1; /*     ( ) */
}

#else /*   */
Event_Number_TypeDef _FirstWaitEvent;							/*    */
Event_Number_TypeDef _WaitEventQueue[Event_Count]; 	/*   */
/*    */
void _SetWaitEvent(Event_Number_TypeDef n)
{
	Event_Number_TypeDef curwait;	
	Event_Number_TypeDef prewait = _FirstWaitEvent;	
	
	if ((_FirstWaitEvent == 255)
		||(_Ev_Prior[_FirstWaitEvent] > _Ev_Prior[n]))	{ /*    */	
		 _WaitEventQueue[n] = _FirstWaitEvent;
		 _FirstWaitEvent = n;
	} else		 			
		for(curwait = _WaitEventQueue[_FirstWaitEvent];;	/*     */
				curwait = _WaitEventQueue[curwait]) 					/*    */	
			if ((curwait == 255)						         /*     */
			||(_Ev_Prior[curwait] > _Ev_Prior[n])) { /*      */
				_WaitEventQueue[n] = curwait;					 /*      */
				_WaitEventQueue[prewait] = n;					 /*      */	
				break; 				
			}
			else prewait = curwait;                  /*   */
			
	_Ev_Stat[n] = EVENT_WAIT;	
	_CountWait_Event += 1; /*     ( ) */
}
#endif

/*****  *******/
ROSA_BASE_TypeDef Event_Planner(Event_Number_TypeDef n)
{	
	Event_Number_TypeDef r = 255;
	
	if (n < Event_Count) {		 /*   */
		
		switch(_Ev_Stat[n]){
			
			case EVENT_NULL: /*    */	
			case EVENT_STOP:				
					if((_CurRun_Event == 255)||
						(_Ev_Prior[n] < _Ev_Prior[_CurRun_Event])) /*     */
						
						switch(_PreRun_Event){										 /*     */
							
							default:																				/*   ,   */
								if(_Ev_Prior[n] >= _Ev_Prior[_PreRun_Event]){	/*    ,   */
									_SetWaitEvent(n);	 													/*   */
									break;
								} 
								else	
									_SetWaitEvent(_PreRun_Event);						 		/*       ))) */	
									
							case  255:																			/*   */						
								_Ev_Stat[n] = EVENT_READY;										/*    */
								_PreRun_Event = n;											
								r = n; 																				/*   */
								break;
							
						}					
					else  /*       */					
						_SetWaitEvent(n);	 	/*   */
					
					break;
					
					
			case EVENT_READY: 				/*       */
				_FailCount(n);
				_PreRun_Event = n;
				r = n;
				break;
						
			
			case EVENT_WAIT:					/*   ,*/					
			case EVENT_BLOCK_WAIT:		/*     */				
			case EVENT_RUN: 				/*    */
				_FailCount(n);
				break;
				
				
			case EVENT_BLOCK:
				_Ev_Stat[n] = EVENT_BLOCK_WAIT;				
				break;
		}
	}		
	
	return r;	
}


/*   */
ROSA_BASE_TypeDef Event_Beginner(void)
{
	if ((_PreRun_Event < Event_Count) &&
		(_Ev_Stat[_PreRun_Event] == EVENT_READY)){
			_Ev_Stat[_PreRun_Event] = EVENT_RUN;			
			_LastRun_Event = _CurRun_Event; /*   */
			_CurRun_Event  = _PreRun_Event;	/*   */
			_PreRun_Event = 255;						
			return _CurRun_Event;
	}	
	return 255;
}	


/*     */
#ifndef ROSA_WaitEvent_QUEUE /*   */
Event_Number_TypeDef __inline WaitEventPicker (void) 
{	
	Event_Number_TypeDef i = _Scan_Wait_Event;
	Event_Number_TypeDef cur_hend  = 255; 
	Event_Priorety_TypeDef cur_prior;
	
	if (_CurRun_Event == 255) cur_prior	 = 255; 			/*   , .  255 */ 
	else cur_prior	= _Ev_Prior[_CurRun_Event];				/*     */
	
	if (cur_prior > _MaxWait_Event_Prior) /*      */
	{	
		_CountWait_Event = 0; 	/*   */
				
		/*   -    */	
		for(i = i; i< Event_Count; i++) {
			if(_Ev_Stat[i] != EVENT_WAIT) 	/*       */
						continue;
						
			if (cur_prior > _Ev_Prior[i]) {				
				cur_hend  = i;
				_MaxWait_Event_Prior = cur_prior;  /*  .   
																						   */					
				cur_prior = _Ev_Prior[i];				
			}	
			else if (_MaxWait_Event_Prior > _Ev_Prior[i]) 
				_MaxWait_Event_Prior = _Ev_Prior[i];
			
			_CountWait_Event += 1; /*     ( ) */
		}

		/*   -   */		
		if (cur_hend != 255) {
			if (_Scan_Wait_Event == cur_hend)
				_Scan_Wait_Event = cur_hend+1;		/*        */
			_Ev_Stat[cur_hend] = EVENT_READY;				
			_PreRun_Event = cur_hend;
			
			_CountWait_Event -= 1; /*     ( ) */
		}
	}
	
	return cur_hend;	
}


#else /*    */
Event_Number_TypeDef __inline WaitEventPicker (void) 
{	
	Event_Number_TypeDef cur_hend  = 255; 
	
	if ((_FirstWaitEvent != 255)			/*      */ 
		&&((_CurRun_Event == 255)				/*     */
	  ||(_Ev_Prior[_CurRun_Event] > _Ev_Prior[_FirstWaitEvent]))) /*       */
	{
		cur_hend = _FirstWaitEvent;
		_FirstWaitEvent = _WaitEventQueue[cur_hend];
		_Ev_Stat[cur_hend] = EVENT_READY;				
		_PreRun_Event = cur_hend;		
		_CountWait_Event -= 1; /*     ( ) */
	}
	
	return cur_hend;	
}
#endif

/*   */
Event_Number_TypeDef Event_Closer(Event_Number_TypeDef n, Event_Number_TypeDef l)
{
	if (_Ev_Stat[n] == EVENT_RUN)	/*      */
		_Ev_Stat[n] = EVENT_STOP; 	/*     */
	_CurRun_Event = l; 				/*    */	
	
	return WaitEventPicker();      /*     */	
}


/*   */
void ROSA_Init(IRQ_Priority_TypeDef p)
{
	ROSA_InitInt(p);
	
	_CurRun_Event = 255;            /*   .  */
	_PreRun_Event = 255;						/*     */
	
#ifdef ROSA_WaitEvent_QUEUE 			/*    */
	_FirstWaitEvent = 255;        /*    */
#endif
	
	ROSA_Critical_Section_End();	/*    */
		
	ROSA_CallEvent(255);			/*     */	
}

